@@ -22,17 +22,17 @@
 -- > import qualified Codec.Archive.Tar.Entry as Tar
 --
 -----------------------------------------------------------------------------
+
+{-# LANGUAGE CPP #-}
+
 module Codec.Archive.Tar.Entry (
 
   -- * Tar entry and associated types
-  Entry(..),
-  --TODO: should be the following with the Entry constructor not exported,
-  --      but haddock cannot document that properly
-  --      see http://trac.haskell.org/haddock/ticket/3
-  --Entry(filePath, fileMode, ownerId, groupId, fileSize, modTime,
-  --      fileType, linkTarget, headerExt, fileContent),
+  GenEntry(..),
+  Entry,
   entryPath,
-  EntryContent(..),
+  GenEntryContent(..),
+  EntryContent,
   Ownership(..),
 
   FileSize,
@@ -47,6 +47,8 @@ module Codec.Archive.Tar.Entry (
   simpleEntry,
   fileEntry,
   directoryEntry,
+  longLinkEntry,
+  longSymLinkEntry,
 
   -- * Standard file permissions
   -- | For maximum portability when constructing archives use only these file
@@ -58,6 +60,10 @@ module Codec.Archive.Tar.Entry (
   -- * Constructing entries from disk files
   packFileEntry,
   packDirectoryEntry,
+  packSymlinkEntry,
+#if __GLASGOW_HASKELL__ >= 908
+  {-# DEPRECATED "The re-export will be removed in future releases of tar, use directory-ospath-streaming package directly " #-}
+#endif
   getDirectoryContentsRecursive,
 
   -- * TarPath type
@@ -73,8 +79,8 @@ module Codec.Archive.Tar.Entry (
   fromLinkTarget,
   fromLinkTargetToPosixPath,
   fromLinkTargetToWindowsPath,
-
   ) where
 
 import Codec.Archive.Tar.Types
 import Codec.Archive.Tar.Pack
+import System.Directory.OsPath.Streaming (getDirectoryContentsRecursive)
