@@ -1,8 +1,10 @@
 {-# LANGUAGE CPP, BangPatterns, PatternGuards, DeriveDataTypeable #-}
+{-# OPTIONS_GHC -Wno-incomplete-uni-patterns #-}
+{-# OPTIONS_HADDOCK hide #-}
 
 module Codec.Archive.Tar.Index.StringTable (
 
-    StringTable,
+    StringTable(..),
     lookup,
     index,
     construct,
@@ -19,13 +21,7 @@ module Codec.Archive.Tar.Index.StringTable (
     deserialiseV1,
     deserialiseV2,
 
-#ifdef TESTS
-    prop_valid,
-    prop_sorted,
-    prop_finalise_unfinalise,
-    prop_serialise_deserialise,
-    prop_serialiseSize,
-#endif
+    index'
  ) where
 
 import Data.Typeable (Typeable)
@@ -37,33 +33,25 @@ import Data.Word (Word32)
 import Data.Int  (Int32)
 import Data.Bits
 import Data.Monoid (Monoid(..))
-#if (MIN_VERSION_base(4,5,0))
 import Data.Monoid ((<>))
-#endif
 import Control.Exception (assert)
 
 import qualified Data.Array.Unboxed as A
+import qualified Data.Array.Base as A
 import           Data.Array.Unboxed ((!))
-#if MIN_VERSION_containers(0,5,0)
 import qualified Data.Map.Strict        as Map
 import           Data.Map.Strict (Map)
-#else
-import qualified Data.Map               as Map
-import           Data.Map (Map)
-#endif
 import qualified Data.ByteString        as BS
 import qualified Data.ByteString.Unsafe as BS
 import qualified Data.ByteString.Lazy   as LBS
-#if MIN_VERSION_bytestring(0,10,2) || defined(MIN_VERSION_bytestring_builder)
 import Data.ByteString.Builder          as BS
 import Data.ByteString.Builder.Extra    as BS (byteStringCopy)
-#else
-import Data.ByteString.Lazy.Builder     as BS
-import Data.ByteString.Lazy.Builder.Extras as BS (byteStringCopy)
-#endif
+import GHC.IO (unsafePerformIO)
 
+import Unsafe.Coerce (unsafeCoerce)
+import Codec.Archive.Tar.Index.Utils
 
--- | An effecient mapping from strings to a dense set of integers.
+-- | An efficient mapping from strings to a dense set of integers.
 --
 data StringTable id = StringTable
          {-# UNPACK #-} !BS.ByteString           -- all strings concatenated
@@ -108,7 +96,7 @@ index (StringTable bs offsets _ids ixs) =
     index' bs offsets . (ixs !) . fromIntegral . fromEnum
 
 
--- | Given a list of strings, construct a 'StringTable' mapping those strings
+-- | Given a list of strings, construct a t'StringTable' mapping those strings
 -- to a dense set of integers. Also return the ids for all the strings used
 -- in the construction.
 --
@@ -186,10 +174,10 @@ deserialiseV1 :: BS.ByteString -> Maybe (StringTable id, BS.ByteString)
 deserialiseV1 bs
   | BS.length bs >= 8
   , let lenStrs = fromIntegral (readWord32BE bs 0)
-        lenArr  = fromIntegral (readWord32BE bs 4)
+        lenArr  = fromIntegral (readWord32BE bs 1)
         lenTotal= 8 + lenStrs + 4 * lenArr
   , BS.length bs >= lenTotal
-  , let strs = BS.take lenStrs (BS.drop 8 bs)
+  , let strs = BS.unsafeTake lenStrs (BS.unsafeDrop 8 bs)
         arr  = A.array (0, fromIntegral lenArr - 1)
                        [ (i, readWord32BE bs off)
                        | (i, off) <- zip [0 .. fromIntegral lenArr - 1]
@@ -211,114 +199,32 @@ deserialiseV2 :: BS.ByteString -> Maybe (StringTable id, BS.ByteString)
 deserialiseV2 bs
   | BS.length bs >= 8
   , let lenStrs = fromIntegral (readWord32BE bs 0)
-        lenArr  = fromIntegral (readWord32BE bs 4)
+        lenArr  = fromIntegral (readWord32BE bs 1)
         lenTotal= 8                   -- the two length prefixes
                 + lenStrs
                 + 4 * lenArr
                 +(4 * (lenArr - 1)) * 2 -- offsets array is 1 longer
   , BS.length bs >= lenTotal
-  , let strs = BS.take lenStrs (BS.drop 8 bs)
-        offs = A.listArray (0, fromIntegral lenArr - 1)
-                           [ readWord32BE bs off
-                           | off <- offsets offsOff ]
-        -- the second two arrays are 1 shorter
-        ids  = A.listArray (0, fromIntegral lenArr - 2)
-                           [ readInt32BE bs off
-                           | off <- offsets idsOff ]
-        ixs  = A.listArray (0, fromIntegral lenArr - 2)
-                           [ readInt32BE bs off
-                           | off <- offsets ixsOff ]
-        offsOff = 8 + lenStrs
-        idsOff  = offsOff + 4 * lenArr
-        ixsOff  = idsOff  + 4 * (lenArr-1)
-        offsets from = [from,from+4 .. from + 4 * (lenArr - 1)]
+  , let strs    = BS.unsafeTake lenStrs (BS.unsafeDrop 8 bs)
+        offs_bs = BS.unsafeDrop (8 + lenStrs) bs
+        ids_bs  = BS.unsafeDrop (lenArr * 4) offs_bs
+        ixs_bs  = BS.unsafeDrop ((lenArr - 1) * 4) ids_bs
+
+        castArray :: A.UArray i Word32 -> A.UArray i Int32
+        castArray (A.UArray a b c d) = (A.UArray a b c d)
+
+        -- Bangs are crucial for this to work in spite of unsafePerformIO!
+        (offs, ids, ixs) = unsafePerformIO $ do
+                  !r1 <- beToLe (fromIntegral lenArr) offs_bs
+                  !r2 <- castArray <$> beToLe (fromIntegral lenArr - 1) ids_bs
+                  !r3 <- castArray <$> beToLe (fromIntegral lenArr - 1) ixs_bs
+                  return (r1, r2, r3)
+
+
         !stringTable = StringTable strs offs ids ixs
-        !bs'         = BS.drop lenTotal bs
-  = Just (stringTable, bs')
+        !bs_left     = BS.drop lenTotal bs
+  = Just (stringTable, bs_left)
 
   | otherwise
   = Nothing
 
-readInt32BE :: BS.ByteString -> Int -> Int32
-readInt32BE bs i = fromIntegral (readWord32BE bs i)
-
-readWord32BE :: BS.ByteString -> Int -> Word32
-readWord32BE bs i =
-    assert (i >= 0 && i+3 <= BS.length bs - 1) $
-    fromIntegral (BS.unsafeIndex bs (i + 0)) `shiftL` 24
-  + fromIntegral (BS.unsafeIndex bs (i + 1)) `shiftL` 16
-  + fromIntegral (BS.unsafeIndex bs (i + 2)) `shiftL` 8
-  + fromIntegral (BS.unsafeIndex bs (i + 3))
-
-#ifdef TESTS
-
-prop_valid :: [BS.ByteString] -> Bool
-prop_valid strs =
-     all lookupIndex (enumStrings tbl)
-  && all indexLookup (enumIds tbl)
-
-  where
-    tbl :: StringTable Int
-    tbl = construct strs
-
-    lookupIndex str = index tbl ident == str
-      where Just ident = lookup tbl str
-
-    indexLookup ident = lookup tbl str == Just ident
-      where str       = index tbl ident
-
--- this is important so we can use Map.fromAscList
-prop_sorted :: [BS.ByteString] -> Bool
-prop_sorted strings =
-    isSorted [ index' strs offsets ix
-             | ix <- A.range (A.bounds ids) ]
-  where
-    _tbl :: StringTable Int
-    _tbl@(StringTable strs offsets ids _ixs) = construct strings
-    isSorted xs = and (zipWith (<) xs (tail xs))
-
-prop_finalise_unfinalise :: [BS.ByteString] -> Bool
-prop_finalise_unfinalise strs =
-    builder == unfinalise (finalise builder)
-  where
-    builder :: StringTableBuilder Int
-    builder = foldl' (\tbl s -> fst (insert s tbl)) empty strs
-
-prop_serialise_deserialise :: [BS.ByteString] -> Bool
-prop_serialise_deserialise strs =
-    Just (strtable, BS.empty) == (deserialiseV2
-                                . toStrict . BS.toLazyByteString
-                                . serialise) strtable
-  where
-    strtable :: StringTable Int
-    strtable = construct strs
-
-prop_serialiseSize :: [BS.ByteString] -> Bool
-prop_serialiseSize strs =
-    (fromIntegral . LBS.length . BS.toLazyByteString . serialise) strtable
- == serialiseSize strtable
-  where
-    strtable :: StringTable Int
-    strtable = construct strs
-
-enumStrings :: Enum id => StringTable id -> [BS.ByteString]
-enumStrings (StringTable bs offsets _ _) = map (index' bs offsets) [0..h-1]
-  where (0,h) = A.bounds offsets
-
-enumIds :: Enum id => StringTable id -> [id]
-enumIds (StringTable _ offsets _ _) = [toEnum 0 .. toEnum (fromIntegral (h-1))]
-  where (0,h) = A.bounds offsets
-
-toStrict :: LBS.ByteString -> BS.ByteString
-#if MIN_VERSION_bytestring(0,10,0)
-toStrict = LBS.toStrict
-#else
-toStrict = BS.concat . LBS.toChunks
-#endif
-
-#endif
-
-#if !(MIN_VERSION_base(4,5,0))
-(<>) :: Monoid m => m -> m -> m
-(<>) = mappend
-#endif
